=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2006-05-19 07:57:09 +0000
+++ bzrlib/bzrdir.py	2006-05-30 12:47:26 +0000
@@ -931,6 +931,9 @@
     _formats = {}
     """The known formats."""
 
+    _control_formats = set()
+    """The core control formats - .bzr, ...."""
+
     _lock_file_name = 'branch-lock'
 
     # _lock_class must be set in subclasses to the lock type, typ.
@@ -938,7 +941,18 @@
 
     @classmethod
     def find_format(klass, transport):
-        """Return the format registered for URL."""
+        """Return the format present at transport."""
+        for format in klass._control_formats:
+            try:
+                return format.probe_transport(transport)
+            except errors.NotBranchError:
+                # this format does not find a control dir here.
+                pass
+        raise errors.NotBranchError(path=transport.base)
+
+    @classmethod
+    def probe_transport(klass, transport):
+        """Return the .bzrdir style transport present at URL."""
         try:
             format_string = transport.get(".bzr/branch-format").read()
             return klass._formats[format_string]
@@ -1044,6 +1058,17 @@
         klass._formats[format.get_format_string()] = format
 
     @classmethod
+    def register_control_format(klass, format):
+        """Register a format that does not use '.bzrdir' for its control dir.
+
+        TODO: This should be pulled up into a 'ControlDirFormat' base class
+        which BzrDirFormat can inherit from, and renamed to register_format 
+        there. It has been done without that for now for simplicity of
+        implementation.
+        """
+        klass._control_formats.add(format)
+
+    @classmethod
     def set_default_format(klass, format):
         klass._default_format = format
 
@@ -1055,6 +1080,14 @@
         assert klass._formats[format.get_format_string()] is format
         del klass._formats[format.get_format_string()]
 
+    @classmethod
+    def unregister_control_format(klass, format):
+        klass._control_formats.remove(format)
+
+
+# register BzrDirFormat as a control format
+BzrDirFormat.register_control_format(BzrDirFormat)
+
 
 class BzrDirFormat4(BzrDirFormat):
     """Bzr dir format 4.

=== modified file 'bzrlib/tests/test_bzrdir.py'
--- bzrlib/tests/test_bzrdir.py	2006-04-18 22:41:16 +0000
+++ bzrlib/tests/test_bzrdir.py	2006-05-30 12:47:26 +0000
@@ -465,6 +465,55 @@
             bzrdir.BzrDirFormat.set_default_format(old_format)
 
 
+class NotBzrDir(bzrlib.bzrdir.BzrDir):
+    """A non .bzr based control directory."""
+
+    def __init__(self, transport, format):
+        self._format = format
+        self.root_transport = transport
+        self.transport = transport.clone('.not')
+
+
+class NotBzrDirFormat(bzrlib.bzrdir.BzrDirFormat):
+    """A test class representing any non-.bzr based disk format."""
+
+    def initialize_on_transport(self, transport):
+        """Initialize a new .not dir in the base directory of a Transport."""
+        transport.mkdir('.not')
+        return self.open(transport)
+
+    def open(self, transport):
+        """Open this directory."""
+        return NotBzrDir(transport, self)
+
+    @classmethod
+    def probe_transport(self, transport):
+        """Our format is present if the transport ends in '.not/'."""
+        if transport.base.endswith('.not/'):
+            return NotBzrDirFormat()
+
+
+class TestNotBzrDir(TestCaseWithTransport):
+    """Tests for using the bzrdir api with a non .bzr based disk format.
+    
+    If/when one of these is in the core, we can let the implementation tests
+    verify this works.
+    """
+
+    def test_create_and_find_format(self):
+        # create a .notbzr dir 
+        format = NotBzrDirFormat()
+        dir = format.initialize(self.get_url())
+        self.assertIsInstance(dir, NotBzrDir)
+        # now probe for it.
+        bzrlib.bzrdir.BzrDirFormat.register_control_format(format)
+        try:
+            found = bzrlib.bzrdir.BzrDirFormat.find_format(
+                get_transport(self.get_url()))
+        finally:
+            bzrlib.bzrdir.BzrDirFormat.unregister_control_format(format)
+
+
 class NonLocalTests(TestCaseWithTransport):
     """Tests for bzrdir static behaviour on non local paths."""
 

